
'''
PACT DUOM ED448 BUILD-PUBLIC-KEY \
--private-key-path "ED448_PRIVATE_KEY.DER" \
--public-key-path "ED448_PUBLIC_KEY.DER"
'''
def BUILD_PUBLIC_KEY (GROUP):
	import click
	@GROUP.command ("BUILD-PUBLIC-KEY")
	@click.option ('--private-key-path', default = '', help = '')
	@click.option ('--public-key-path', default = '', help = '')
	def BUILD_PUBLIC_KEY (private_key_path, public_key_path):
		PRIVATE_KEY_PATH = private_key_path
		PUBLIC_KEY_PATH = public_key_path
	
		from DUOM.ED448.PUBLIC_KEY.BUILD import BUILD_PUBLIC_KEY
		[ PUBLIC_KEY, PUBLIC_KEY_EXPORT ] = BUILD_PUBLIC_KEY (
			PRIVATE_KEY_PATH,
			PUBLIC_KEY_PATH
		)

	return;




'''
import requests
import json
r = requests.put (
	'http://127.0.0.1:1110', 
	data = json.dumps ({
		"START": "BUILD ED448 PRIVATE KEY",
		"FIELDS": {
			"SEED": "5986888b11358bf3d541b41eea5daece1c6eff64130a45fc8b9ca48f3e0e02463c99c5aedc8a847686d669b7d547c18fe448fc5111ca88f4e8",
			"PATH": "ED448_PRIVATE_KEY.DER"
		}
	})
)
print (r.text)
'''
def BUILD_PRIVATE_KEY (
	FIELDS
):
	SEED = FIELDS ["SEED"]
	PATH = FIELDS ["PATH"]

	#print ("SEED:", SEED)
	#print ("PATH:", PATH)
	
	from DUOM.ED448.PRIVATE_KEY.CREATE import CREATE_PRIVATE_KEY
	PRIVATE_KEY = CREATE_PRIVATE_KEY (
		SEED, 
		"DER",
		PATH = PATH 
	)
	if (PRIVATE_KEY ["GOOD"] == False):
		return {
			"GOOD": False
		}
	
	return {
		"GOOD": True
	}





'''
PACT DUOM ED448 VERIFY \
--public-key-path "ED448_PUBLIC_KEY.DER" \
--unsigned-utf8-path "UNSIGNED_MESSAGE.UTF8" \
--signed-bytes-path "SIGNED_MESSAGE.BYTES"
'''
def VERIFY (GROUP):
	import click
	@click.option ('--public-key-path', default = '', help = '')
	@click.option ('--unsigned-utf8-path', default = '', help = '')
	@click.option ('--signed-bytes-path', default = '', help = '')
	@GROUP.command ("VERIFY")
	def VERIFY (
		public_key_path, 
		unsigned_utf8_path,
		signed_bytes_path
	):	
		from DUOM.ED448.PUBLIC_KEY.SCAN import SCAN_PUBLIC_KEY
		[ PUBLIC_KEY, PUBLIC_KEY_BYTES, PUBLIC_KEY_STRING ] = SCAN_PUBLIC_KEY (public_key_path)

		from UTF8.SCAN import SCAN_UTF8_NOTE
		UNSIGNED_BYTES = SCAN_UTF8_NOTE (unsigned_utf8_path) [1]

		print ("?????????????????")

		from BYTES.SCAN import SCAN_BYTES
		SIGNED_BYTES = SCAN_BYTES (signed_bytes_path) [0]

		from DUOM.ED448.VERIFY import VERIFY
		VERIFIED = VERIFY (
			PUBLIC_KEY, 
			UNSIGNED_BYTES, 
			SIGNED_BYTES
		)

		if (VERIFIED):
			exit (0)
			
		exit (99)
		

		return;	






import qrcode
from io import BytesIO
from base64 import b64encode

def GENERATE_QR (
	STRING = ''
):
	QR_CODE_IMG = qrcode.make (STRING)
	
	BUFFER = BytesIO ()
	QR_CODE_IMG.save (BUFFER)
	
	BUFFER.seek (0)
	ENCODED_IMG = b64encode (BUFFER.read ()).decode ()
	
	QR_CODE_BASE_64 = f'data:image/png;base64,{ENCODED_IMG}'
	
	return QR_CODE_BASE_64;









#
#	https://setuptools.pypa.io/en/latest/userguide/quickstart.html
#
#	https://github.com/pypa/sampleproject/blob/db5806e0a3204034c51b1c00dde7d5eb3fa2532e/setup.py
#
from setuptools import setup, find_packages


def SCAN_VARIABLES ():
	import json

	VARIABLES = {}
	try:
		with open ('VARIABLES.JSON') as f:
			STRING = f.read ().strip()
			print (STRING)
		
			VARIABLES = json.loads (STRING)
	except Exception as E:
		print (E)
	
		pass;
		
	return VARIABLES;


def SCAN_DESCRIPTION ():
	DESCRIPTION = ''
	try:
		with open ('ACADEMY.rst') as f:
			DESCRIPTION = f.read ()
		print (DESCRIPTION)
	except Exception as E:
		pass;
		
	return DESCRIPTION;

VARIABLES = SCAN_VARIABLES ()
print (VARIABLES)
print (type (VARIABLES))

from glob import glob

STRUCTURE = 'STRUCTURE/MODULES/' + VARIABLES ['NAME']


setup (
    name = VARIABLES ['NAME'],
    version = VARIABLES ['VERSION'],
    install_requires = VARIABLES ['INSTALL_REQUIRES'],	
	
	package_dir = { 
		VARIABLES ['NAME']: STRUCTURE
	},
	package_data = {
		#VARIABLES ['NAME']: [ 
		#	'STRUCTURE/MODULES/SCRIPTS/ENCHUFLA' 
		#],
		'STRUCTURE/MODULES': [ "*.HTML" ]
		
		#"": [ "*.txt" ]
    },
	
	
	scripts = [ 
		'STRUCTURE/MODULES/SCRIPTS/ENCHUFLA' 
	],
	
	license = "SSDL",
	long_description = SCAN_DESCRIPTION (),
	long_description_content_type = "text/plain"
)





'''
	from BYTES.SCAN import SCAN_BYTES
	[ BYTES ] = SCAN_BYTES ()
'''

from fractions import Fraction

def SCAN_BYTES (PATH):
	with open (PATH, mode = 'rb') as NOTE:
		BYTES 	= NOTE.read ()
		STRING 	= BYTES.hex ()
		
		print (len (STRING))
		print ("BYTE COUNT = ", Fraction (len (STRING), 2))
		
		
		return [ BYTES ];





from .STARTS import STARTS

from flask import Flask, request
import json

from QR.GENERATE import GENERATE_QR

RECORDS = 0

def TAP (
	PORT = 0
):
	app = Flask (__name__)

	'''
		STRATEGY:
			ENCHUFLA SEND "{ 'START': 'BUILD ED448 PRIVATE KEY', 'FIELDS': { 'PATH': 'ED448_PRIVATE_KEY.DER' } }"
	'''

	'''
		curl -X PUT localhost:5000 -d "{ 'START': 'BUILD ED448 PRIVATE KEY', 'FIELDS': { 'PATH': 'ED448_PRIVATE_KEY.DER' } }"
		curl -X PUT localhost:5000 -d '{ "START": "BUILD ED448 PRIVATE KEY" }'
		curl -X PUT localhost:5000 -d '{ "START": "BUILD ED448 PRIVATE KEY", "FIELDS": { "PATH": "ED448_PRIVATE_KEY.DER", "SEED": "4986888B11358BF3D541B41EEA5DAECE1C6EFF64130A45FC8B9CA48F3E0E02463C99C5AEDC8A847686D669B7D547C18FE448FC5111CA88F4E8" } }'
	'''
	@app.route ("/", methods = [ 'PUT' ])
	def ROUTE ():
		DATA = ""
		
		try:
			DATA = request.get_data ();
			if (RECORDS >= 1): print ("--- DATA ::", DATA)
			
			UTF8 = DATA.decode ('utf-8')
			if (RECORDS >= 1): print ("UTF8 ::", UTF8)
			
			JSON = json.loads (UTF8)
			if (RECORDS >= 1): print ("JSON ::", json.dumps (UTF8))
			if (RECORDS >= 1): print ("JSON ::", type (JSON))

			START = JSON ["START"]
			if (RECORDS >= 1): print ("START ::", START)
			
			FIELDS = JSON ["FIELDS"]
			if (RECORDS >= 1): print ("FIELDS ::", FIELDS)
			
			START = STARTS (
				START = START,
				FIELDS = FIELDS
			)
			if (START ["GOOD"] == True):
				return json.dumps ({ "GOOD": True }), 999
			
			ALARM = ""
			if ("ALARM" in START):
				ALARM = START ["ALARM"]
		
			return json.dumps ({ "GOOD": False, "ALARM": ALARM }), 701
			
		except Exception as E:
			import traceback
			traceback.print_exception (E)
		
		
		return json.dumps ({ "GOOD": False }), 702

	
	
	app.run (
		port = PORT
	)
	
	return;
	
	'''
		curl -X PUT localhost:5000 -d "{  }"
		
		curl -X PUT localhost:5000 -d "{}" -H "Content-Type: application/json"
	'''
	@app.route ("/", methods = [ 'PUT' ])
	def BUILD_PRIVATE_KEY ():
		#data = request.form.get ('data')
		data = request.get_data ()

		print ("DATA:", request.data)
	
		#from DUOM.ED448.PRIVATE_KEY.BUILD import BUILD_PRIVATE_KEY
		#[ PRIVATE_KEY, PRIVATE_KEY_EXPORT ] = BUILD_PRIVATE_KEY (SEED, FORMAT)
	
		return "<p>PUT</p>"	
	
	@app.route ("/QR")
	def QR ():
		QR_CODE = GENERATE_QR (
			STRING = 'THIS IS A QR CODE MESSAGE!'
		)
	
		return f"""
<body style="background: #222">
	<img src='{ QR_CODE }' />
</body>
		"""
		
	app.run ()





'''
	from DUOM.ED448.VERIFY import VERIFY
	VERIFY (
		PUBLIC_KEY, 
		UNSIGNED_BYTES, 
		SIGNED_BYTES
	)
'''

from Crypto.Signature 	import eddsa

def VERIFY (
	PUBLIC_KEY, 
	UNSIGNED_BYTES, 
	SIGNED_BYTES
):
	# PUBLIC_KEY = ECC.import_key (PUBLIC_KEY_STRING)
	VERIFIER = eddsa.new (PUBLIC_KEY, 'rfc8032')
	
	try:
		VERIFIER.verify (UNSIGNED_BYTES, SIGNED_BYTES)
		print ("VERIFICATION SUCCESSFUL")
		
		return True;
		
	except ValueError:
		print ("VERIFICATION FAILED")
		
	return False;




def ED448 (GROUP):
	import click
	@GROUP.group ("ED448")
	def GROUP ():
		pass
	
	from .BUILD_PRIVATE_KEY import BUILD_PRIVATE_KEY
	BUILD_PRIVATE_KEY (GROUP)
	
	from .BUILD_PUBLIC_KEY import BUILD_PUBLIC_KEY
	BUILD_PUBLIC_KEY (GROUP)
	
	from .SIGN import SIGN
	SIGN (GROUP)
	
	from .VERIFY import VERIFY
	VERIFY (GROUP)

	return;